#include <unistd.h>
#include <utime.h>
#include <errno.h>

#include <assert.h>
#include <iostream>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <rlog/rlog.h>

#include "File.hpp"

File::File(const struct stat *st) :
	m_fd (-1),
	m_inode (st->st_ino),
	m_refs (0)
{
}

File::~File()
{
	assert (m_refs == 0);
	assert (m_fd == -1);
}

int File::getattr(const char *name, struct stat *st)
{
	int r;

	m_FileName = name;

	r = ::lstat(name, st);

	return r;
}

int File::unlink(const char *name)
{
	int r;

	r = ::unlink(name);

	return r;
}

int File::truncate(const char *name, off_t size)
{
	int r;

	r = ::truncate(name, size);

	return r;
}

int File::utime(const char *name, struct utimbuf *buf)
{
	int r;

	r = ::utime(name, buf);

	return r;
}

ssize_t File::read(char *buf, size_t size, off_t offset)
{
	ssize_t r;
	
	r = ::pread(m_fd, buf, size, offset);

	return r;
}

ssize_t File::write(const char *buf, size_t size, off_t offset)
{
	ssize_t r;

	r = ::pwrite(m_fd, buf, size, offset);

	return r;
}

int File::open_force(const char *name)
{
	int		r;
	int		fd;
	int		tmp_errno = 0;
	struct stat	buf;

	if (::stat(name, &buf) == -1)
		return -1;

	if (::chmod(name, buf.st_mode | 0600) == -1)
		return -1;

	fd = ::open(name, O_RDWR);
	if (fd == -1)
		tmp_errno = errno;

	r = ::chmod(name, buf.st_mode);
	assert (r != -1);

	if (fd == -1)
		errno = tmp_errno;
	
	return fd;
}

int File::open(const char *name)
{
	int fd;
	
	fd = ::open(name, O_RDWR);
	
	if ((fd == -1) && (errno == EACCES || errno == EPERM))
	{
		fd = open_force(name);
	}
	return fd;
}

int File::open(const char *name, int flags)
{
	// Use only one file descriptor for one file determined
	// by inode number.
	//
	if (m_fd != -1)
	{
		++m_refs;
		
		return m_fd;
	}
	
	m_fd = open(name);
	
	if (m_fd != -1)
	{
		++m_refs;
	}

	rDebug("File::open file '%s', inode %ld",
		name, (long int) m_inode);

	return m_fd;
}

int File::flush(const char *name)
{
	return 0;
}

int File::release(const char *name)
{
	if (--m_refs == 0)
	{
		::close(m_fd);
		
		m_fd = -1;
	}
	assert (m_refs >= 0);

	return 0;
}

int File::fdatasync(const char *name)
{
	return 0;
}

int File::fsync(const char *name)
{
	return 0;
}

